#! /usr/bin/perl
#---------------------------------------------------------------------

use Test::More tests => 1;

BEGIN {
    use_ok('PostScript::ScheduleGrid');
}

diag("Testing PostScript::ScheduleGrid $PostScript::ScheduleGrid::VERSION");
